import "Turbine";
import "Turbine.UI";
import "Turbine.UI.Lotro";

pluginData = Turbine.PluginData.Load(Turbine.DataScope.Character, 'OPL') or {
	plugins = {}, order = {},
}

PluginWindow = class( Turbine.UI.Lotro.Window);

function PluginWindow:Constructor()
	Turbine.UI.Lotro.Window.Constructor( self );

	selectedIndex = 1;
	
	self.Width = 600;
	self.Height = 500;
	self.disableResize = true;

	local displayWidth, displayHeight = Turbine.UI.Display.GetSize();

	-- configure the main window
	self:SetPosition( ( displayWidth - self.Width ) / 2, ( displayHeight - self.Height ) / 2 );
	self:SetSize( self.Width, self.Height );
	self:SetBackColor( Turbine.UI.Color(0.0,0,0,0) );
	self:SetText( "OPL Plugin Window" );
	self:SetOpacity( 1 );
	self:SetVisible( true );

	listbox = Turbine.UI.ListBox();
      	listbox:SetParent( self );
    	listbox:SetBackColor( Turbine.UI.Color( 0.1, 0.1, 0.1 ) );
     	listbox:SetPosition( 30, 50 );
      	listbox:SetSize( 200, 400 );

	scrollBar = listbox:GetVerticalScrollBar();
	if(scrollBar == nil ) then
		scrollBar = Turbine.UI.Lotro.ScrollBar();
	end
	scrollBar:SetBackColor(Turbine.UI.Color(0.87,0.1,0.1,0.1));
	scrollBar:SetOrientation( Turbine.UI.Orientation.Vertical );	
	scrollBar:SetSize( 10, 400 );
	scrollBar:SetPosition( 190,0);
	scrollBar:SetZOrder(100);
	scrollBar:SetVisible( true );
	scrollBar:SetParent(listbox);

	listbox:SetVerticalScrollBar(scrollBar);

	availablePlugins = Turbine.PluginManager.GetAvailablePlugins();
      
	for num, totalPlugins in pairs (availablePlugins) do
      		for k, v in pairs( pluginData.order ) do
			if k == num then
          			 local listItem = Turbine.UI.Label();
         			 listItem:SetSize( 150, 20 );
		 		 listItem:SetText( v );
		 		 listItem.MouseClick = function(sender, args)
					listbox:GetItem(selectedIndex):SetBackColor(Turbine.UI.Color(0.87,0.1,0.1,0.1));
           				sender:SetBackColor(Turbine.UI.Color(0.95,0.1,0.1,0.6));
           				selectedIndex = listbox:IndexOfItem(sender);
        			  end
	          	listbox:InsertItem( k, listItem );
			end
		end
        end

	listbox2 = Turbine.UI.ListBox();
	listbox2:SetParent( self );
    	listbox2:SetBackColor( Turbine.UI.Color( 0.1, 0.1, 0.1 ) );
     	listbox2:SetPosition( 370, 50 );
      	listbox2:SetSize( 200, 400 );
	listbox2:SetZOrder(100);
	listbox2:SetVisible( true);

	scrollBar2 = listbox2:GetVerticalScrollBar();
	if(scrollBar2 == nil ) then
		scrollBar2 = Turbine.UI.Lotro.ScrollBar();
	end
	scrollBar2:SetBackColor(Turbine.UI.Color(0.87,0.1,0.1,0.1));
	scrollBar2:SetOrientation( Turbine.UI.Orientation.Vertical );	
	scrollBar2:SetParent(listbox2);
	scrollBar2:SetSize( 10, 400 );
	scrollBar2:SetPosition( 190,0);
	scrollBar2:SetZOrder(100);
	scrollBar2:SetVisible( true );

	listbox2:SetVerticalScrollBar(scrollBar2);

      	i = 0;

	for k, v in pairs (availablePlugins) do
	  local control1 = Turbine.UI.Control();
	  control1:SetSize( 150, 20 );
	  control1:SetPosition(0, 25 * i);
	  control1:SetVisible(true);

          local listItem = Turbine.UI.Lotro.CheckBox();
          listItem:SetSize( 150, 20 );
	  listItem:SetPosition(0, 0);
	  isChecked = false;
	  for k2, v2 in pairs (pluginData.plugins) do
		if(k2 == v.Name and v2 == true) then
			isChecked = true;
		end
	  end
	  listItem:SetChecked( isChecked );
	  listItem:SetText( v.Name );
	  listItem:SetParent(control1);
	  listItem:SetWantsUpdates(true);
	  listItem:SetVisible(true);

	  listItem.CheckedChanged = function(sender, args)
		--write('CheckedChanged')
		if sender:IsChecked() then
			pluginData.plugins[sender:GetText()] = true;
			local listItem1 = Turbine.UI.Label();
          		listItem1:SetSize( 150, 20 );
	  		listItem1:SetText( sender:GetText() );
			listItem1.MouseClick = function(sender, args)
					listbox:GetItem(selectedIndex):SetBackColor(Turbine.UI.Color(0.87,0.1,0.1,0.1));
           				sender:SetBackColor(Turbine.UI.Color(0.95,0.1,0.1,0.6));
           				selectedIndex = listbox:IndexOfItem(sender);
        			  end
			listbox:AddItem( listItem1 );
			pluginData.order[listbox:IndexOfItem(listItem1)] = sender:GetText();
			SavePlugin();
		else
			itemOrderNumber = nil;
			for k1, v1 in pairs (pluginData.order) do
				if v1 == sender:GetText() then
					itemOrderNumber = k1;
				end
			end
			if itemOrderNumber ~= nil then
				listbox:RemoveItemAt(itemOrderNumber);
			end
			if selectedIndex > listbox:GetItemCount() then
				selectedIndex = 1;
			end
			if itemOrderNumber < selectedIndex then
				selectedIndex = selectedIndex - 1;
			end
			pluginData.plugins[sender:GetText()] = nil;
			pluginData.order[itemOrderNumber] = nil;
			fixPluginOrderList(itemOrderNumber, listbox);
			SavePlugin();
		end
	  end
          
	  listbox2:AddItem( control1 );
	  i = i + 1;
        end

	moveUpButton = Turbine.UI.Button();
   	moveUpButton:SetSize(120,20);
   	moveUpButton:SetPosition(240, 50);
	moveUpButton:SetBackColor(Turbine.UI.Color(0.95,0.1,0.1,0.6));
   	moveUpButton:SetText("Move Plugin Up");
   	moveUpButton:SetParent( self );
   	moveUpButton:SetVisible(true);
	
	moveUpButton.Click = function(sender, args)
		selectedItemHolder = listbox:GetItem(selectedIndex);
		listbox:RemoveItemAt(selectedIndex);
		selectedIndex = selectedIndex - 1;
		if selectedIndex == 0 then
			selectedIndex = 1;
		end
		if selectedIndex > listbox:GetItemCount() then
			selectedIndex = selectedIndex - 1;
		end
		listbox:InsertItem(selectedIndex, selectedItemHolder);
		fixPluginData(listbox);
	end
   
   	moveDownButton = Turbine.UI.Button();
   	moveDownButton:SetSize(120,20);
   	moveDownButton:SetPosition(240, 80);
	moveDownButton:SetBackColor(Turbine.UI.Color(0.95,0.1,0.1,0.6));
   	moveDownButton:SetText("Move Plugin Down");
	moveDownButton:SetParent( self );
   	moveDownButton:SetVisible(true);

	moveDownButton.Click = function(sender, args)
		selectedItemHolder = listbox:GetItem(selectedIndex);
		listbox:RemoveItemAt(selectedIndex);
		selectedIndex = selectedIndex + 1;
		if selectedIndex == 0 then
			selectedIndex = 1;
		end
		if selectedIndex > listbox:GetItemCount() + 1 then
			selectedIndex = selectedIndex - 1;
		end
		listbox:InsertItem(selectedIndex, selectedItemHolder);
		fixPluginData(listbox);
	end

	reloadButton = Turbine.UI.Button();
   	reloadButton:SetSize(120,20);
   	reloadButton:SetPosition(240, 400);
	reloadButton:SetBackColor(Turbine.UI.Color(0.95,0.1,0.1,0.6));
   	reloadButton:SetText("Re/load Plugins");
	reloadButton:SetParent( self );
   	reloadButton:SetVisible(true);

	reloadButton.Click = function(sender, args)
		for k, v in pairs (Turbine.PluginManager.GetLoadedPlugins()) do
			if v.ScriptState ~= "SDRPlugins" then
				Turbine.PluginManager.UnloadScriptState(v.ScriptState);
			end
		end
		for pluginNumToLoad = 1, listbox:GetItemCount() do
			for k, v in pairs (pluginData.order) do
				if k == pluginNumToLoad then
					Turbine.PluginManager.LoadPlugin(v);
				end
			end
		end
	end
end

-- Saves the plugin
function SavePlugin()
	Turbine.PluginData.Save( Turbine.DataScope.Character, 'OPL', pluginData );
end

-- Fixes the numbers so they continue to be equal to the Index in the ItemList
function fixPluginOrderList(pluginIndexNumber, listBox)
	for loop = pluginIndexNumber + 1, listBox:GetItemCount() + 1 do
		for k, v in pairs (pluginData.order) do
			if k > pluginIndexNumber and k == loop then
				pluginData.order[k - 1] = v;
			end
			if k > listBox:GetItemCount() and k == loop then
				pluginData.order[k] = nil;
			end
		end
	end
end

-- Fixes the order of the list after swapping two items (updates the plugindata file to be correct)
function fixPluginData(listBox)
	for i = 1, listBox:GetItemCount() do
		pluginData.order[i] = listBox:GetItem(i):GetText();
	end
	SavePlugin();
end